<?php
/**
 * Build email signatures for use in Apple Mail
 *
 * see https://dmblog.com/2013/10/html-signatures-for-apple-mail-and-iphone/ by Daniel Menjivar
 * 
 */
class DM_Email_Signature {

	/**
	 * @var string	The message ID that Apple assigns to the email signature
	 **/
	private $message_id;

	/**
	 * @var string	The character set to use for the email signature
	 **/
	private $charset;

	/**
	 * @var string	The font family(ies) to use for the css of the email signature
	 **/
	private $font_family;

	/**
	 * @var string	The html content of the email signature
	 **/
	private $html_content;

	/**
	 * Create an instance of our email signature object.  This way, we can call our static method and chain properties all in one line.
	 *
	 *		$signature = DM_Email_Signature::factory();
	 *
	 * @return	$this
	 **/
	public static function factory()
	{
		$class = __CLASS__;
		return new $class;
	}

	/**
	 * A magic method to set the values of properties, but only if they exist
	 *
	 *		DM_Email_Signature::factory()->color(array(0,0,0));
	 *		DM_Email_Signature::factory()->events(TRUE);
	 *
	 * @param	string	The name of the method that was called (color, etc)
	 * @param	string	The arguments that were sent with the requested method
	 * @return	$this
	 **/
	public function __call($name, $arguments)
	{
		// check if the property exists
		 if (property_exists($this, $name))
		 {
			// if there are no arguments/parameters then we should throw an exception
		 	if (empty($arguments))
		 	{
		 		throw new InvalidArgumentException('You must specify a value for the '.$name.' property in '.get_class($this));
			// otherwise if we have arguments, SET the value
		 	} else {
		 		$this->$name	= $arguments[0];

				// return $this to allow chaining
		 		return $this;
		 	}
		 // if the property doesn't exist, we should throw an exception
		 } else {
		 	throw new BadMethodCallException('The property "'.$name.'" does not exist in '.get_class($this));
		 }
	}

	/**
	 * Render our email signature
	 *
	 *		DM_Email_Signature::factory()->render();
	 *
	 * @return	string	The content of the email signature
	 **/
	public function render()
	{
		$headers	 = "Content-Transfer-Encoding: quoted-printable\r\n";
		$headers	.= "Content-Type: text/html;\r\n\tcharset=".$this->charset."\r\n";
		$headers	.= "Message-Id: <".$this->message_id.">\r\n";
		$headers	.= "Mime-Version: 1.0 (Mac OS X Mail 6.6 \(1510\))\r\n\r\n";

		$html		 = '
			<body style="color:rgb(0,0,0);font-family:'.$this->font_family.';font-size:medium;font-style:normal;font-variant:normal;font-weight:normal;letter-spacing:normal;line-height:normal;orphans:2;text-align:-webkit-auto;text-indent:0px;text-transform:none;white-space:normal;widows:2;word-spacing:0px;-webkit-text-size-adjust:auto;-webkit-text-stroke-width:0px;word-wrap:break-word;-webkit-nbsp-mode:space;-webkit-line-break:after-white-space;">
				<br class="Apple-interchange-newline">
				<div>'.$this->html_content.'</div>
			</body>
			<br class="Apple-interchange-newline">';

		$rendered		 = $html."\r\n\r\n<br>\r\n<pre>";
		$rendered		.= htmlentities($headers.quoted_printable_encode($this->minify($html))).'=';
		$rendered		.= "</pre>\r\n<br><br>";

		return $rendered;
	}

	/**
	 * Remove unnecessary characters from our HTML to make it smaller
	 *
	 *		$minified_text = $this->minify($text);
	 *
	 * @return	string	The text input with certain characters (new lines, tabs) removed
	 **/
	private function minify($html)
	{
		$replacements = array(
							'	'	=> '',	// get rid of tabs
							PHP_EOL	=> ''	// get rid of new lines
		);

		return str_replace(array_keys($replacements),array_values($replacements),(string) $html);
	}
} // end of DM_Email_Signature class

?>
<!DOCTYPE html>
<html dir="ltr" lang="en-US">
<head>
	<meta charset="utf-8" />
	<title>Daniel's Email Signature Generator</title>
	<style>
		body{font-family:Helvetica;font-size:14px}
		#content{margin:0;padding:30px;width:950px}
		#signature-form{margin:0;width:310px;float:left}
		fieldset{padding:20px}
		label{display:block;color:#787878;font-size:12px}
		input{width:250px;font-size:18px}
		#main-footer{clear:both;position:fixed;bottom:0;text-align:center;width:100%;color:#787878}
		#main-footer a {text-decoration:none;color:#787878}
		#main-footer a:hover{text-decoration:underline;color:#00508c}
		#render{margin:20px;font-size:18px;padding:15px}
		#right{margin:0;width:610px;float:right}
	</style>
</head>
<body>
<div id="top">
	<section id="content" role="main">
		<div id="signature-form">
			<form action="signatures.php" method="post">
				<fieldset>
					<p>
						<label for="message_id">Message ID :</label>
						<input type="text" name="message_id" id="message_id" value="">
					</p>
					<p>
						<label for="charset">Character set :</label>
						<input type="text" name="charset" id="charset" value="utf-8">
					</p>
					<p>
						<label for="font_family">CSS Font Family :</label>
						<input type="text" name="font_family" id="font_family" value="Helvetica,Arial,sans-serif">
					</p>
					<p>
						<label for="html_content">HTML Content :</label>
						<textarea name="html_content" id="html_content" cols="33" rows="7"></textarea>
					</p>
					<p>
						<button type="submit" id="render">Generate Signature!</button>
					</p>
				</fieldset>
			</form>
		</div>
		<div id="right"><?php

			if (empty($_POST))
			{?>
				<p>For more information on how to use this signature generator correctly, please read <a href="https://dmblog.com/2013/10/html-signatures-for-apple-mail-and-iphone/?utm_source=signature%20generator&utm_medium=plugins&utm_content=signature%20generator%20help&utm_campaign=signature%20generator">dmblog.com/2013/10/html-signatures-for-apple-mail-and-iphone/</a></p><?php
			} else {
				extract($_POST);
				$message_id		 	= (! empty($message_id)) ? $message_id : '******MESSAGE ID HERE******';
				$charset			= (! empty($charset)) ? $charset : 'utf-8';
				$font_family		= (! empty($font_family)) ? $font_family : 'Helvetica,Arial,sans-serif';
				$email_signature	= DM_Email_Signature::factory()
										->message_id($message_id)
										->charset($charset)
										->font_family($font_family)
										->html_content($html_content)
										->render();
				echo "\r\n\r\n".$email_signature."\r\n\r\n";
			}?>
		</div>
	</section>
	<footer id="main-footer" role="contentinfo">
		<p><small>&copy; 2012 — <?php echo date('Y');?> <a href="https://dmblog.com/?utm_source=signature%20generator&utm_medium=plugins&utm_content=signature%20generator%20footer&utm_campaign=signature%20generator" rel="external" title="dmblog.com">Daniel Menjivar</a> — All Rights Reserved.</small></p>
	</footer>
</div>
</body>
</html>